from typing import Sequence, Collection
from numba import njit
import numpy as np


@njit
def get_ss_w(sqdistances: np.ndarray, labels: np.ndarray, bc: np.ndarray):
    sum = np.zeros_like(bc, dtype=np.float64)

    for d, lbs in zip(sqdistances, labels):
        if lbs[0] == lbs[1]:
            sum[lbs[0]] += d
    return np.sum(sum / bc)


def get_f(ss_t: np.float64, ss_w: np.float64, a: int, n: int):
    ss_a = ss_t - ss_w
    f = (ss_a / (a - 1)) / (ss_w / (n - a))
    return f


def permanova(sqdistances: np.ndarray, fastlabels: np.ndarray):
    packed_labels = np.empty(
        ((len(fastlabels) - 1) * len(fastlabels) // 2, 2), dtype=np.int16
    )
    pc = 0
    for i in range(len(fastlabels)):
        for j in range(i + 1, len(fastlabels)):
            packed_labels[pc][:] = fastlabels[i], fastlabels[j]
            pc += 1

    packed_dists = sqdistances[np.triu_indices_from(sqdistances, 1)]

    bc = np.bincount(fastlabels)
    ss_t = np.sum(packed_dists) / len(fastlabels)
    ss_w = get_ss_w(packed_dists, packed_labels, bc)

    f = get_f(ss_t, ss_w, len(bc), len(fastlabels))
    other_fs = []
    for _ in range(1000):
        np.random.shuffle(packed_dists)
        other_fs.append(
            get_f(
                ss_t,
                get_ss_w(packed_dists, packed_labels, bc),
                len(bc),
                len(fastlabels),
            )
        )

    return f, (np.array(other_fs) >= f).mean()



from __future__ import annotations
from typing import Any
from reference import square
import peroxymanova
import numpy as np
from scipy.spatial import distance_matrix
from scipy.stats import f_oneway

size = 100
objects = np.random.random((size, 1))
dist: np.ndarray[np.float64, Any] = distance_matrix(objects, objects)
labels = np.random.randint(0, 2, size)


def test_it():
    python = square.permanova(dist**2, labels.copy())
    our = peroxymanova.permanova(dist**2, labels.copy())
    anova = f_oneway(*(objects[labels == i] for i in np.unique(labels)))
    run_py_results = peroxymanova.run(objects, lambda a, b: np.sqrt(np.sum((a-b)**2)), labels, 'python')
    run_sci_results = peroxymanova.run(objects, 'euclid', labels, 'scipy')


    assert np.allclose(anova.statistic[0], our[0])
    assert np.allclose(anova.statistic[0], run_py_results.statistic)
    assert np.allclose(anova.statistic[0], run_sci_results.statistic)



import numpy as np
from typing import Any

def permanova(
    sqdistances: np.ndarray[np.float64, Any], labels: np.ndarray[np.uint, Any]
) -> tuple[float, float]: ...
def ordinal_encoding(arr: np.ndarray[np.str_, Any]) -> np.ndarray[np.uint, Any]: ...



import sys
import tomllib
import os


def get_project_version():
    try:
        with open("pyproject.toml", "rb") as toml_file:
            data = tomllib.load(toml_file)
        version = data["project"]["version"]
        return version
    except Exception as e:
        print(f"Error reading pyproject.toml: {e}")
        sys.exit(1)


def compare_versions(old_version, project_version):
    old_version_parts = [int(part) for part in old_version.split(".")]
    project_version_parts = [int(part) for part in project_version.split(".")]
    try:
        for old, new in zip(old_version_parts, project_version_parts, strict=True):
            if new < old:
                return ""
            if new > old:
                return project_version
        return ""
    except ValueError:
        print(f"version formats incompatible: {old_version = }, {project_version = }")
        sys.exit(1)


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python version_check.py <latest_tag>")
        sys.exit(1)

    old_version = sys.argv[1]
    if old_version.startswith("v"):
        old_version = old_version[1:]

    project_version = get_project_version()

    new_version = compare_versions(old_version, project_version)

    print(f"{new_version = }")
    with open(os.environ["GITHUB_ENV"], "a") as f:
        f.write("new_version=" + new_version)
